@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: codecs.h,v $
+ * Revision 1.67  2003/05/27 09:39:15  dereksmithies
+ * Add frames per sec variable to the videocodec class
+ *
  * Revision 1.66  2002/12/16 09:11:15  robertj
  * Added new video bit rate control, thanks Walter H. Whitlock
  *
@@ -946,7 +949,7 @@ class H323VideoCodec : public H323Codec
        This message is sent via the H245 Logical Channel.
     */
     void SendMiscCommand(unsigned command);
- 
+
   protected:
 
     int frameWidth;
@@ -970,7 +973,8 @@ class H323VideoCodec : public H323Codec
     PTimeInterval grabInterval;
     
     int frameNum, packetNum, oldPacketNum;
-    
+    int framesPerSec;
+
     PMutex  videoHandlerActive;    
 };
 
