@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: g729codec.h,v $
+ * Revision 1.7  2003/05/05 11:59:21  robertj
+ * Changed to use autoconf style selection of options and subsystems.
+ *
  * Revision 1.6  2002/11/12 00:06:10  robertj
  * Added check for Voice Age G.729 only being able to do a single instance
  *   of the encoder and decoder. Now fails the second isntance isntead of
@@ -58,6 +61,9 @@
 #pragma interface
 #endif
 
+#include <openh323buildopts.h>
+
+#if VOICE_AGE_G729A
 
 #include "h323caps.h"
 
@@ -202,6 +208,8 @@ class H323_G729ACodec : public H323FramedAudioCodec
 };
 
 
+#endif // VOICE_AGE_G729A
+
 #endif // __OPAL_G729CODEC_H
 
 
