@@ -27,6 +27,32 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: gkserver.h,v $
+ * Revision 1.67  2003/04/30 00:28:50  robertj
+ * Redesigned the alternate credentials in ARQ system as old implementation
+ *   was fraught with concurrency issues, most importantly it can cause false
+ *   detection of replay attacks taking out an endpoint completely.
+ *
+ * Revision 1.66  2003/04/02 06:58:10  robertj
+ * Added support for (optional) duplicate aliases in gatekeeper server, necessitating
+ *   the removal of AddAlias and RemoveAlias functions.
+ *
+ * Revision 1.65  2003/04/02 06:07:12  robertj
+ * Added H.323 Annex G Peer Element support to gatekeeper server.
+ *
+ * Revision 1.64  2003/04/01 07:30:52  robertj
+ * Added function to translate alias into alias list and transport to make
+ *   use of full information returned by peer element.
+ *
+ * Revision 1.63  2003/04/01 04:47:48  robertj
+ * Abstracted H.225 RAS transaction processing (RIP and secondary thread) in
+ *   server environment for use by H.501 peer elements.
+ *
+ * Revision 1.62  2003/03/26 07:10:03  robertj
+ * Added more access functions.
+ *
+ * Revision 1.61  2003/03/17 22:52:06  craigs
+ * Improvided useability of AddAlias and RemoveAlias
+ *
  * Revision 1.60  2003/03/04 05:34:13  robertj
  * Fixed GNU warnings
  *
@@ -274,11 +300,12 @@ class H323RegisteredEndPoint;
 class H323GatekeeperListener;
 class H323GatekeeperServer;
 class H323RasPDU;
+class H323PeerElement;
 
 
-class H323GatekeeperRequest : public PObject
+class H323GatekeeperRequest : public H323Transaction
 {
-    PCLASSINFO(H323GatekeeperRequest, PObject);
+    PCLASSINFO(H323GatekeeperRequest, H323Transaction);
   public:
   /**@name Construction */
   //@{
@@ -290,61 +317,30 @@ class H323GatekeeperRequest : public PObject
     );
   //@}
 
-    enum Response {
-      Ignore = -2,
-      Reject = -1,
-      Confirm = 0
-    };
-    inline static Response InProgress(unsigned time) { return (Response)(time&0xffff); }
+    virtual H323TransactionPDU * CreateRIP(
+      unsigned sequenceNumber,
+      unsigned delay
+    ) const;
 
-    BOOL HandlePDU();
-    BOOL WritePDU(
-      H323RasPDU & pdu
+    virtual BOOL WritePDU(
+      H323TransactionPDU & pdu
     );
-
+    BOOL CheckCryptoTokens();
     BOOL CheckGatekeeperIdentifier();
     BOOL GetRegisteredEndPoint();
-    BOOL CheckCryptoTokens();
-    BOOL CheckCryptoTokens(
-      const H235Authenticators & authenticators
-    );
 
-#if PTRACING
-    virtual const char * GetName() const = 0;
-#endif
     virtual PString GetGatekeeperIdentifier() const = 0;
     virtual unsigned GetGatekeeperRejectTag() const = 0;
     virtual PString GetEndpointIdentifier() const = 0;
     virtual unsigned GetRegisteredEndPointRejectTag() const = 0;
-    virtual H235Authenticator::ValidationResult ValidatePDU() const = 0;
     virtual unsigned GetSecurityRejectTag() const = 0;
-    virtual void SetRejectReason(
-      unsigned reasonCode
-    ) = 0;
 
-    BOOL IsFastResponseRequired() const { return fastResponseRequired; }
-    H323TransportAddress GetReplyAddress() const { return replyAddresses[0]; }
-    const H323TransportAddressArray & GetReplyAddresses() const { return replyAddresses; }
-    BOOL IsBehindNAT() const { return isBehindNAT; }
     H323GatekeeperListener & GetRasChannel() const { return rasChannel; }
-    H235Authenticator::ValidationResult GetAuthenticatorResult() const { return authenticatorResult; }
 
     PSafePtr<H323RegisteredEndPoint> endpoint;
 
   protected:
-    virtual Response OnHandlePDU() = 0;
-    PDECLARE_NOTIFIER(PThread, H323GatekeeperRequest, SlowHandler);
-
     H323GatekeeperListener & rasChannel;
-    unsigned                 requestSequenceNumber;
-    H323TransportAddressArray replyAddresses;
-    H323RasPDU               request;
-    H323RasPDU               confirm;
-    H323RasPDU               reject;
-    BOOL                     fastResponseRequired;
-
-    H235Authenticator::ValidationResult authenticatorResult;
-    BOOL                                isBehindNAT;
 };
 
 
@@ -989,6 +985,10 @@ class H323RegisteredEndPoint : public PSafeObject
       */
     const PString & GetIdentifier() const { return identifier; }
 
+    /**Get the Peer Element descriptor ID assigned to the endpoint.
+      */
+    const OpalGloballyUniqueID & GetDescriptorID() const { return descriptorID; }
+
     /**Get the gatekeeper server data object that owns this endpoint.
       */
     H323GatekeeperServer & GetGatekeeper() const { return gatekeeper; }
@@ -1010,6 +1010,11 @@ class H323RegisteredEndPoint : public PSafeObject
       PINDEX idx
     ) const { return rasAddresses[idx]; }
 
+    /**Get the addresses that can be used to contact this
+       endpoint via the H.225/Q.931 protocol, ie normal calls.
+      */
+    const H323TransportAddressArray & GetSignalAddresses() const { return signalAddresses; }
+
     /**Get the number of addresses that can be used to contact this
        endpoint via the H.225/Q.931 protocol, ie normal calls.
       */
@@ -1022,6 +1027,10 @@ class H323RegisteredEndPoint : public PSafeObject
       PINDEX idx
     ) const { return signalAddresses[idx]; }
 
+    /**Get the aliases this endpoint may be identified by.
+      */
+    const PStringArray & GetAliases() const { return aliases; }
+
     /**Get the number of aliases this endpoint may be identified by.
       */
     PINDEX GetAliasCount() const { return aliases.GetSize(); }
@@ -1078,6 +1087,7 @@ class H323RegisteredEndPoint : public PSafeObject
     H323GatekeeperListener  * rasChannel;
 
     PString                   identifier;
+    OpalGloballyUniqueID      descriptorID;
     H323TransportAddressArray rasAddresses;
     H323TransportAddressArray signalAddresses;
     PStringArray              aliases;
@@ -1354,17 +1364,6 @@ class H323GatekeeperServer : public H323TransactionServer
       H323RegisteredEndPoint * ep
     );
 
-    /**Remove an alias from the server database.
-       A registered endpoint is searched for and the alias removed from that
-       endpoint. If there are no more aliases left for the endpoint then the
-       registered endpoint itself is removed from the database.
-
-       Returns TRUE if the endpoint itself was removed.
-      */
-    virtual BOOL RemoveAlias(
-      const PString & alias
-    );
-
     /**Create a new registered endpoint object.
        The user woiuld not usually use this function as it is used internally
        by the server when new registration requests (RRQ) are received.
@@ -1525,9 +1524,26 @@ class H323GatekeeperServer : public H323TransactionServer
       H323GatekeeperLRQ & request
     );
 
+    /**Translate a given alias to an alias set and signal address.
+       This is called by the OnAdmission() handler to fill in the ACF
+       or OnLocation() to fill the LCF informing the calling endpoint where
+       to actually connect to.
+
+       It is expected that a user would override this function to implement
+       application specified look up algorithms.
+
+       The default behaviour calls TranslateAliasAddressToSignalAddress().
+      */
+    virtual BOOL TranslateAliasAddress(
+      const H225_AliasAddress & alias,
+      H225_ArrayOf_AliasAddress & aliases,
+      H323TransportAddress & address
+    );
+
     /**Translate a given alias to a signal address.
        This is called by the OnAdmission() handler to fill in the ACF
-       informing the calling endpoint where to actually connect to.
+       or OnLocation() to fill the LCF informing the calling endpoint where
+       to actually connect to.
 
        It is expected that a user would override this function to implement
        application specified look up algorithms.
@@ -1625,10 +1641,8 @@ class H323GatekeeperServer : public H323TransactionServer
        The default behavour does nothing and returns FALSE.
       */
     virtual BOOL GetAdmissionRequestAuthentication(
-      H323GatekeeperARQ & info,  /// ARQ being constructed
-      PString & remoteId,        /// Remote ID to be modified
-      PString & localId,         /// Local ID to be modified
-      PString & password         /// Password to be modified
+      H323GatekeeperARQ & info,           /// ARQ being constructed
+      H235Authenticators & authenticators /// New authenticators for ARQ
     );
 
     /**Get password for user if H.235 security active.
@@ -1642,6 +1656,40 @@ class H323GatekeeperServer : public H323TransactionServer
     ) const;
   //@}
 
+  /**@name H.323 Annex G Peer Element support */
+  //@{
+    /**Get the associated peer element for the gatekeeper.
+      */
+    H323PeerElement * GetPeerElement() const { return peerElement; }
+
+    /**Set the associated peer element for the gatekeeper.
+       The existing peer element is automatically deleted.
+      */
+    void SetPeerElement(
+      H323PeerElement * newPeerElement
+    );
+
+    /**Create an associated peer element for the gatekeeper.
+       The existing peer element is checked to be listening on the same
+       interface as specified and if it is not it is changed to the specified
+       address.
+      */
+    void CreatePeerElement(
+      const H323TransportAddress & h501Interface
+    );
+
+    /**Create a new Peer Element and establish a service relationship.
+       If append is FALSE, the existing peer element is automatically deleted
+       and recreated. If append is TRUE then a new service relationship is
+       added to the existing peer element.
+      */
+    BOOL OpenPeerElement(
+      const H323TransportAddress & remotePeer,
+      BOOL append = FALSE,
+      BOOL keepTrying = TRUE
+    );
+  //@}
+
   /**@name Access functions */
   //@{
     /**Get the identifier name for this gatekeeper.
@@ -1732,6 +1780,12 @@ class H323GatekeeperServer : public H323TransactionServer
   //@}
 
   protected:
+    //Internal function to remove an alias from the server database.
+    void InternalRemoveAlias(
+      H323RegisteredEndPoint & ep,
+      const PString & alias
+    );
+
     PDECLARE_NOTIFIER(PThread, H323GatekeeperServer, MonitorMain);
 
     // Configuration & policy variables
@@ -1743,6 +1797,7 @@ class H323GatekeeperServer : public H323TransactionServer
     unsigned defaultTimeToLive;
     unsigned defaultInfoResponseRate;
     BOOL     overwriteOnSameSignalAddress;
+    BOOL     canHaveDuplicateAlias;
     BOOL     canOnlyCallRegisteredEP;
     BOOL     canOnlyAnswerRegisteredEP;
     BOOL     answerCallPreGrantedARQ;
@@ -1764,6 +1819,8 @@ class H323GatekeeperServer : public H323TransactionServer
     PLIST(ListenerList, H323GatekeeperListener);
     ListenerList listeners;
 
+    H323PeerElement * peerElement;
+
     PSafeDictionary<PString, H323RegisteredEndPoint> byIdentifier;
 
     class StringMap : public PString {
