@@ -25,6 +25,9 @@
  *                 Derek Smithies (derek@indranet.co.nz)
  *
  * $Log: h261codec.h,v $
+ * Revision 1.32  2003/04/03 23:54:11  robertj
+ * Added fast update to H.261 codec, thanks Gustavo Garca Bernardo
+ *
  * Revision 1.31  2002/12/16 09:11:15  robertj
  * Added new video bit rate control, thanks Walter H. Whitlock
  *
@@ -356,6 +359,11 @@ class H323_H261Codec : public H323VideoCodec
     */
     void SetBackgroundFill(int fillLevel);
 
+    /**Process a FastUpdatePicture command from remote endpoint.
+       The default behaviour does nothing.
+     */
+    virtual void OnFastUpdatePicture();
+
     /**Process a request for a new frame, 
        as part of the picture has been lost.
     */
