@@ -27,6 +27,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: ixjlid.h,v $
+ * Revision 1.63  2003/04/29 08:27:47  robertj
+ * Cleaned up documentation for new wink duration functions.
+ *
+ * Revision 1.62  2003/04/28 01:47:53  dereks
+ * Add ability to set/get wink duration for ixj device.
+ *
  * Revision 1.61  2002/11/06 04:03:38  dereks
  * Improve docs for  SetToneFilterParameters().
  *
@@ -547,8 +553,7 @@ class OpalIxJDevice : public OpalLineInterfaceDevice
       unsigned & volume   /// Volume level from 0 to 100%
     );
 
-
-    /**Set acoustic echo cancellation.
+    /**Get acoustic echo cancellation.
       */
     AECLevels GetAEC(
       unsigned line    /// Number of line
@@ -558,9 +563,23 @@ class OpalIxJDevice : public OpalLineInterfaceDevice
       */
     BOOL SetAEC(
       unsigned line,    /// Number of line
-      AECLevels level  /// AEC level
+      AECLevels level   /// AEC level
     );
 
+    /**Get wink detect minimum duration.
+       This is the signal used by telcos to end PSTN call.
+      */
+    unsigned GetWinkDuration(
+      unsigned line    /// Number of line
+    );
+
+    /**Set wink detect minimum duration.
+       This is the signal used by telcos to end PSTN call.
+      */
+    BOOL SetWinkDuration(
+      unsigned line,        /// Number of line
+      unsigned winkDuration /// New minimum duration
+    );
 
     /**Get voice activity detection.
        Note, not all devices, or selected codecs, may support this function.
