@@ -27,6 +27,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: rtp.h,v $
+ * Revision 1.49  2003/05/14 13:46:39  rjongbloed
+ * Removed hack of using special payload type for H.263 for a method which
+ *   would be less prone to failure in the future.
+ *
+ * Revision 1.48  2003/05/02 04:57:43  robertj
+ * Added header extension support to RTP data frame class.
+ *
+ * Revision 1.47  2003/05/02 04:21:53  craigs
+ * Added hacked OpalH263 codec
+ *
  * Revision 1.46  2003/02/07 00:30:41  robertj
  * Changes for bizarre usage of RTP code outside of scope of H.323 specs.
  *
@@ -243,6 +253,7 @@ class RTP_DataFrame : public PBYTEArray
       H263,         // H.263
 
       LastKnownPayloadType,
+
       DynamicBase = 96,
       MaxPayloadType = 127,
       IllegalPayloadType
@@ -272,7 +283,13 @@ class RTP_DataFrame : public PBYTEArray
     DWORD  GetContribSource(PINDEX idx) const;
     void   SetContribSource(PINDEX idx, DWORD src);
 
-    PINDEX GetHeaderSize()     const { return 12 + 4*GetContribSrcCount(); }
+    PINDEX GetHeaderSize() const;
+
+    int GetExtensionType() const; // -1 is no extension
+    void   SetExtensionType(int type);
+    PINDEX GetExtensionSize() const;
+    BOOL   SetExtensionSize(PINDEX sz);
+    BYTE * GetExtensionPtr() const;
 
     PINDEX GetPayloadSize() const { return payloadSize; }
     BOOL   SetPayloadSize(PINDEX sz);
