@@ -27,6 +27,19 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: transports.h,v $
+ * Revision 1.42  2003/04/10 09:44:55  robertj
+ * Added associated transport to new GetInterfaceAddresses() function so
+ *   interfaces can be ordered according to active transport links. Improves
+ *   interoperability.
+ * Replaced old listener GetTransportPDU() with GetInterfaceAddresses()
+ *   and H323SetTransportAddresses() functions.
+ *
+ * Revision 1.41  2003/04/10 01:03:25  craigs
+ * Added functions to access to lists of interfaces
+ *
+ * Revision 1.40  2003/03/21 05:24:02  robertj
+ * Added setting of remote port in UDP transport constructor.
+ *
  * Revision 1.39  2003/02/06 04:29:23  robertj
  * Added more support for adding things to H323TransportAddressArrays
  *
@@ -356,13 +369,6 @@ class H323Listener : public PThread
       */
     virtual H323TransportAddress GetTransportAddress() const = 0;
 
-    /**Set up a transport address PDU for RAS channel.
-      */
-    virtual void SetUpTransportPDU(
-      H225_ArrayOf_TransportAddress & pdu,  /// List of transport addresses listening on
-      const H323Transport & associatedTransport /// Associated transport for precendence and translation
-    ) = 0;
-
     /**Set up a transport address PDU for bidirectional logical channels.
       */
     virtual BOOL SetUpTransportPDU(
@@ -379,6 +385,30 @@ class H323Listener : public PThread
 PLIST(H323ListenerList, H323Listener);
 
 
+/** Return a list of transport addresses corresponding to a listener list
+  */
+H323TransportAddressArray H323GetInterfaceAddresses(
+  const H323ListenerList & listeners, /// List of listeners
+  BOOL excludeLocalHost = TRUE,       /// Flag to exclude 127.0.0.1
+  H323Transport * associatedTransport = NULL
+                          /// Associated transport for precedence and translation
+);
+
+H323TransportAddressArray H323GetInterfaceAddresses(
+  const H323TransportAddress & addr,  /// Possible INADDR_ANY address
+  BOOL excludeLocalHost = TRUE,       /// Flag to exclude 127.0.0.1
+  H323Transport * associatedTransport = NULL
+                          /// Associated transport for precedence and translation
+);
+
+/**Set the PDU field for the list of transport addresses
+  */
+void H323SetTransportAddresses(
+  const H323Transport & associatedTransport,   /// Transport for NAT address translation
+  const H323TransportAddressArray & addresses, /// Addresses to set
+  H225_ArrayOf_TransportAddress & pdu          /// List of PDU transport addresses
+);
+
 
 /**This class describes a I/O transport protocol..
    A "transport" is an object that listens for incoming connections on the
@@ -561,6 +591,10 @@ class H323Transport : public PIndirectChannel
 
   /**@name Member variable access */
   //@{
+    /**Get the associated endpoint to this transport.
+      */
+    H323EndPoint & GetEndPoint() const { return endpoint; }
+
     /**Attach a thread to the transport.
       */
     void AttachThread(
@@ -677,13 +711,6 @@ class H323ListenerTCP : public H323Listener
       */
     virtual H323TransportAddress GetTransportAddress() const;
 
-    /**Set up a transport address PDU for RAS channel.
-      */
-    virtual void SetUpTransportPDU(
-      H225_ArrayOf_TransportAddress & pdu, /// List of transport addresses listening on
-      const H323Transport & associatedTransport /// Associated transport for precendence and translation
-    );
-
     /**Set up a transport address PDU for bidirectional logical channels.
       */
     virtual BOOL SetUpTransportPDU(
@@ -817,9 +844,10 @@ class H323TransportUDP : public H323TransportIP
     /**Create a new transport channel.
      */
     H323TransportUDP(
-      H323EndPoint & endpoint,                 /// H323 End Point object
-      PIPSocket::Address binding = INADDR_ANY, /// Local interface to listen on
-      WORD port = 0                            /// Local port to listen on
+      H323EndPoint & endpoint,                  /// H323 End Point object
+      PIPSocket::Address binding = INADDR_ANY,  /// Local interface to listen on
+      WORD localPort = 0,                       /// Local port to listen on
+      WORD remotePort = 0                       /// Remote port to connect on
     );
     ~H323TransportUDP();
 
