@@ -25,6 +25,19 @@
  *                 Derek Smithies (derek@indranet.co.nz)
  *
  * $Log: h261codec.cxx,v $
+ * Revision 1.56  2003/04/03 23:54:15  robertj
+ * Added fast update to H.261 codec, thanks Gustavo Garca Bernardo
+ *
+ * Revision 1.55  2003/03/20 23:45:46  dereks
+ * Make formatting more consistant with the openh323.org standard.
+ *
+ * Revision 1.54  2003/03/17 08:05:02  robertj
+ * Removed videoio classes in openh323 as now has versions in pwlib.
+ *
+ * Revision 1.53  2003/03/11 22:05:00  dereks
+ * Video receive will not terminate in abnormal situations
+ * Thanks to Damien Sandras.
+ *
  * Revision 1.52  2002/12/29 22:35:34  dereks
  * Fix so video with Windows XP works. Thanks Damien Sandras.
  *
@@ -207,7 +220,6 @@
 
 #include "h261codec.h"
 
-#include "videoio.h"
 #include "h245.h"
 #include "rtp.h"
 
@@ -451,6 +463,7 @@ BOOL H323_H261Codec::Read(BYTE * buffer,
 
   frameWidth  = ((PVideoChannel *)rawDataChannel)->GetGrabWidth();
   frameHeight = ((PVideoChannel *)rawDataChannel)->GetGrabHeight();
+  PTRACE(6, "H261\tVideo grab size is " << frameWidth << "x" << frameHeight);
 
   if( frameWidth == 0 ) {
     PTRACE(1,"H261\tVideo grab width is 0 x 0, close down video transmission thread.");
@@ -461,7 +474,7 @@ BOOL H323_H261Codec::Read(BYTE * buffer,
   videoEncoder->SetSize(frameWidth, frameHeight);
 
   PINDEX bytesInFrame = 0;
-  BOOL ok=TRUE;
+  BOOL ok = TRUE;
 
 #define NUMAVG 8
 
@@ -560,8 +573,8 @@ BOOL H323_H261Codec::Read(BYTE * buffer,
         << (PTimer::Tick() - grabStartTime).GetMilliSeconds() << " ms.");
       packetNum = 0; // reset packet counter
       // If there is a Renderer attached, display the grabbed video.
-      if( ((PVideoChannel *)rawDataChannel)->IsRenderOpen() ) {
-        ok=RenderFrame(); //use data from grab process.
+      if (((PVideoChannel *)rawDataChannel)->IsRenderOpen() ) {
+        ok = RenderFrame(); //use data from grab process.
       }
 #ifdef INC_ENCODE
       videoEncoder->PreProcessOneFrame(); //Prepare to generate H261 packets
@@ -694,8 +707,11 @@ BOOL H323_H261Codec::Write(const BYTE * buffer,
 
   videoDecoder->mark(now);
   BOOL ok = videoDecoder->decode(header, length, lostPreviousPacket);
-  if (!ok) return FALSE;
-
+  if (!ok) {
+    PTRACE (3, "H261\t Could not decode frame, continuing in hope.");
+    return TRUE;
+  }
+  
   // If the incoming video stream changes size, resize the rendering device.
   ok = Resize(videoDecoder->width(), videoDecoder->height());
 
@@ -718,7 +734,7 @@ BOOL H323_H261Codec::Resize(int _width, int _height)
   //Check for a resize is carried out one two level -.
   // a) size change in the receive video stream.
 
-  if( (frameWidth!=_width) || (frameHeight!=_height) ) {
+  if ((frameWidth != _width) || (frameHeight != _height) ) {
       frameWidth  = _width;
       frameHeight = _height;
 
@@ -726,8 +742,10 @@ BOOL H323_H261Codec::Resize(int _width, int _height)
       delete rvts;
       rvts = new BYTE[nblk];
       memset(rvts, 0, nblk);
-      if(videoDecoder!=NULL)
-  videoDecoder->marks(rvts);
+      if (videoDecoder != NULL) 
+	videoDecoder->marks(rvts);
+      if (rawDataChannel != NULL)
+	((PVideoChannel *)rawDataChannel)->SetRenderFrameSize(_width, _height);
   }
 
   return TRUE;
@@ -764,7 +782,9 @@ BOOL H323_H261Codec::RenderFrame()
 
     //Now display local image.
     ((PVideoChannel *)rawDataChannel)->SetRenderFrameSize(frameWidth, frameHeight);
-    PTRACE(6, "H261\tSize of video rendering frame set to " << frameWidth << "x" << frameHeight);
+    PTRACE(6, "H261\tSize of video rendering frame set to " << 
+	   frameWidth << "x" << frameHeight << 
+	   " for channel:" << ((direction == Encoder) ? "encoding" : "decoding"));
 
     if (direction == Encoder)
         ok = rawDataChannel->Write((const void *)videoEncoder->GetFramePtr(),0);
@@ -813,6 +833,16 @@ void H323_H261Codec::SetBackgroundFill(int idle)
 }
 
 
+void H323_H261Codec::OnFastUpdatePicture()
+{
+  PTRACE(3,"H261\tFastUpdatePicture received");
+  PWaitAndSignal mutex1(videoHandlerActive);
+
+  if ( videoEncoder != NULL )
+    videoEncoder->FastUpdatePicture();
+}
+
+
 void H323_H261Codec::OnLostPartialPicture()
 {
   PTRACE(3,"H261\tLost partial picture message ignored, not implemented");
