@@ -27,6 +27,26 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323pdu.cxx,v $
+ * Revision 1.129  2003/05/29 09:19:52  rjongbloed
+ * Fixed minor problem with including DisplayName IE if localPartyName
+ *   is blank, now does not include it, thanks Auri Vizgaitis
+ *
+ * Revision 1.128  2003/05/06 06:24:16  robertj
+ * Added continuous DTMF tone support (H.245 UserInputIndication - signalUpdate)
+ *   as per header documentation, thanks Auri Vizgaitis
+ *
+ * Revision 1.127  2003/04/10 09:37:20  robertj
+ * Added some more functions for converting to alias addresses.
+ *
+ * Revision 1.126  2003/04/01 03:11:51  robertj
+ * Added function to get array of AliasAddress into PStringArray.
+ *
+ * Revision 1.125  2003/03/25 04:56:21  robertj
+ * Fixed issues to do with multiple inheritence in transaction reply cache.
+ *
+ * Revision 1.124  2003/03/20 01:51:12  robertj
+ * More abstraction of H.225 RAS and H.501 protocols transaction handling.
+ *
  * Revision 1.123  2003/02/25 06:48:19  robertj
  * More work on PDU transaction abstraction.
  *
@@ -506,12 +526,31 @@ void H323TraceDumpPDU(const char * proto,
 
 ///////////////////////////////////////////////////////////////////////////////
 
-void H323SetAliasAddresses(const PStringList & names,
-                           H225_ArrayOf_AliasAddress & aliases)
+void H323SetAliasAddresses(const H323TransportAddressArray & addresses, H225_ArrayOf_AliasAddress & aliases)
+{
+  aliases.SetSize(addresses.GetSize());
+  for (PINDEX i = 0; i < addresses.GetSize(); i++)
+    H323SetAliasAddress(addresses[i], aliases[i]);
+}
+
+
+void H323SetAliasAddresses(const PStringArray & names,
+                           H225_ArrayOf_AliasAddress & aliases,
+                           int tag)
 {
   aliases.SetSize(names.GetSize());
   for (PINDEX i = 0; i < names.GetSize(); i++)
-    H323SetAliasAddress(names[i], aliases[i]);
+    H323SetAliasAddress(names[i], aliases[i], tag);
+}
+
+
+void H323SetAliasAddresses(const PStringList & names,
+                           H225_ArrayOf_AliasAddress & aliases,
+                           int tag)
+{
+  aliases.SetSize(names.GetSize());
+  for (PINDEX i = 0; i < names.GetSize(); i++)
+    H323SetAliasAddress(names[i], aliases[i], tag);
 }
 
 
@@ -521,6 +560,13 @@ static BOOL IsE164(const PString & str)
 }
 
 
+void H323SetAliasAddress(const H323TransportAddress & address, H225_AliasAddress & alias)
+{
+  alias.SetTag(H225_AliasAddress::e_transportID);
+  address.SetPDU(alias);
+}
+
+
 void H323SetAliasAddress(const PString & name, H225_AliasAddress & alias, int tag)
 {
   // See if explicitly specified the alias type, otherwise guess it from
@@ -581,6 +627,17 @@ void H323SetAliasAddress(const PString & name, H225_AliasAddress & alias, int ta
 
 /////////////////////////////////////////////////////////////////////////////
 
+PStringArray H323GetAliasAddressStrings(const H225_ArrayOf_AliasAddress & aliases)
+{
+  PStringArray strings(aliases.GetSize());
+
+  for (PINDEX i = 0; i < aliases.GetSize(); i++)
+    strings[i] = H323GetAliasAddressString(aliases[i]);
+
+  return strings;
+}
+
+
 PString H323GetAliasAddressString(const H225_AliasAddress & alias)
 {
   switch (alias.GetTag()) {
@@ -1340,7 +1397,8 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
     }
   }
   else {
-    q931pdu.SetDisplayName(localName);
+    if (!localName)
+      q931pdu.SetDisplayName(localName);
     for (i = 0; i < aliases.GetSize(); i++) {
       if (IsE164(aliases[i])) {
         number = aliases[i];
@@ -1656,20 +1714,32 @@ H245_UserInputIndication & H323ControlPDU::BuildUserInputIndication(char tone,
 {
   H245_UserInputIndication & ind = Build(H245_IndicationMessage::e_userInput);
 
-  ind.SetTag(H245_UserInputIndication::e_signal);
-  H245_UserInputIndication_signal & sig = ind;
+  if (tone != ' ') {
+    ind.SetTag(H245_UserInputIndication::e_signal);
+    H245_UserInputIndication_signal & sig = ind;
 
-  sig.m_signalType.SetValue(tone);
+    sig.m_signalType.SetValue(tone);
 
-  if (duration > 0) {
-    sig.IncludeOptionalField(H245_UserInputIndication_signal::e_duration);
-    sig.m_duration = duration;
+    if (duration > 0) {
+      sig.IncludeOptionalField(H245_UserInputIndication_signal::e_duration);
+      sig.m_duration = duration;
+    }
+
+    if (logicalChannel > 0) {
+      sig.IncludeOptionalField(H245_UserInputIndication_signal::e_rtp);
+      sig.m_rtp.m_logicalChannelNumber = logicalChannel;
+      sig.m_rtp.m_timestamp = rtpTimestamp;
+    }
   }
+  else {
+    ind.SetTag(H245_UserInputIndication::e_signalUpdate);
+    H245_UserInputIndication_signalUpdate & sig = ind;
 
-  if (logicalChannel > 0) {
-    sig.IncludeOptionalField(H245_UserInputIndication_signal::e_rtp);
-    sig.m_rtp.m_logicalChannelNumber = logicalChannel;
-    sig.m_rtp.m_timestamp = rtpTimestamp;
+    sig.m_duration = duration;
+    if (logicalChannel > 0) {
+      sig.IncludeOptionalField(H245_UserInputIndication_signalUpdate::e_rtp);
+      sig.m_rtp.m_logicalChannelNumber = logicalChannel;
+    }
   }
 
   return ind;
@@ -1723,6 +1793,170 @@ H323RasPDU::H323RasPDU(const H235Authenticators & auth)
 }
 
 
+PObject * H323RasPDU::Clone() const
+{
+  return new H323RasPDU(*this);
+}
+
+
+PASN_Object & H323RasPDU::GetPDU()
+{
+  return *this;
+}
+
+
+PASN_Choice & H323RasPDU::GetChoice()
+{
+  return *this;
+}
+
+
+const PASN_Object & H323RasPDU::GetPDU() const
+{
+  return *this;
+}
+
+
+const PASN_Choice & H323RasPDU::GetChoice() const
+{
+  return *this;
+}
+
+
+unsigned H323RasPDU::GetSequenceNumber() const
+{
+  switch (GetTag()) {
+    case H225_RasMessage::e_gatekeeperRequest :
+      return ((const H225_GatekeeperRequest &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_gatekeeperConfirm :
+      return ((const H225_GatekeeperConfirm &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_gatekeeperReject :
+      return ((const H225_GatekeeperReject &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_registrationRequest :
+      return ((const H225_RegistrationRequest &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_registrationConfirm :
+      return ((const H225_RegistrationConfirm &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_registrationReject :
+      return ((const H225_RegistrationReject &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_unregistrationRequest :
+      return ((const H225_UnregistrationRequest &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_unregistrationConfirm :
+      return ((const H225_UnregistrationConfirm &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_unregistrationReject :
+      return ((const H225_UnregistrationReject &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_admissionRequest :
+      return ((const H225_AdmissionRequest &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_admissionConfirm :
+      return ((const H225_AdmissionConfirm &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_admissionReject :
+      return ((const H225_AdmissionReject &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_bandwidthRequest :
+      return ((const H225_BandwidthRequest &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_bandwidthConfirm :
+      return ((const H225_BandwidthConfirm &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_bandwidthReject :
+      return ((const H225_BandwidthReject &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_disengageRequest :
+      return ((const H225_DisengageRequest &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_disengageConfirm :
+      return ((const H225_DisengageConfirm &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_disengageReject :
+      return ((const H225_DisengageReject &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_locationRequest :
+      return ((const H225_LocationRequest &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_locationConfirm :
+      return ((const H225_LocationConfirm &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_locationReject :
+      return ((const H225_LocationReject &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_infoRequest :
+      return ((const H225_InfoRequest &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_infoRequestResponse :
+      return ((const H225_InfoRequestResponse &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_nonStandardMessage :
+      return ((const H225_NonStandardMessage &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_unknownMessageResponse :
+      return ((const H225_UnknownMessageResponse &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_requestInProgress :
+      return ((const H225_RequestInProgress &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_resourcesAvailableIndicate :
+      return ((const H225_ResourcesAvailableIndicate &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_resourcesAvailableConfirm :
+      return ((const H225_ResourcesAvailableConfirm &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_infoRequestAck :
+      return ((const H225_InfoRequestAck &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_infoRequestNak :
+      return ((const H225_InfoRequestNak &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_serviceControlIndication :
+      return ((const H225_ServiceControlIndication &)*this).m_requestSeqNum;
+
+    case H225_RasMessage::e_serviceControlResponse :
+      return ((const H225_ServiceControlResponse &)*this).m_requestSeqNum;
+
+    default :
+      return 0;
+  }
+}
+
+
+unsigned H323RasPDU::GetRequestInProgressDelay() const
+{
+  if (GetTag() != H225_RasMessage::e_requestInProgress)
+    return 0;
+
+  return ((const H225_RequestInProgress &)*this).m_delay;
+}
+
+
+#if PTRACING
+const char * H323RasPDU::GetProtocolName() const
+{
+  return "H225RAS";
+}
+#endif
+
+
+H323TransactionPDU * H323RasPDU::ClonePDU() const
+{
+  return new H323RasPDU(*this);
+}
+
+
+void H323RasPDU::DeletePDU()
+{
+  delete this;
+}
+
+
 H225_GatekeeperRequest & H323RasPDU::BuildGatekeeperRequest(unsigned seqNum)
 {
   SetTag(e_gatekeeperRequest);
@@ -2011,109 +2245,4 @@ H225_ServiceControlResponse & H323RasPDU::BuildServiceControlResponse(unsigned s
 }
 
 
-unsigned H323RasPDU::GetSequenceNumber() const
-{
-  switch (GetTag()) {
-    case H225_RasMessage::e_gatekeeperRequest :
-      return ((const H225_GatekeeperRequest &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_gatekeeperConfirm :
-      return ((const H225_GatekeeperConfirm &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_gatekeeperReject :
-      return ((const H225_GatekeeperReject &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_registrationRequest :
-      return ((const H225_RegistrationRequest &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_registrationConfirm :
-      return ((const H225_RegistrationConfirm &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_registrationReject :
-      return ((const H225_RegistrationReject &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_unregistrationRequest :
-      return ((const H225_UnregistrationRequest &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_unregistrationConfirm :
-      return ((const H225_UnregistrationConfirm &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_unregistrationReject :
-      return ((const H225_UnregistrationReject &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_admissionRequest :
-      return ((const H225_AdmissionRequest &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_admissionConfirm :
-      return ((const H225_AdmissionConfirm &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_admissionReject :
-      return ((const H225_AdmissionReject &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_bandwidthRequest :
-      return ((const H225_BandwidthRequest &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_bandwidthConfirm :
-      return ((const H225_BandwidthConfirm &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_bandwidthReject :
-      return ((const H225_BandwidthReject &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_disengageRequest :
-      return ((const H225_DisengageRequest &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_disengageConfirm :
-      return ((const H225_DisengageConfirm &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_disengageReject :
-      return ((const H225_DisengageReject &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_locationRequest :
-      return ((const H225_LocationRequest &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_locationConfirm :
-      return ((const H225_LocationConfirm &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_locationReject :
-      return ((const H225_LocationReject &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_infoRequest :
-      return ((const H225_InfoRequest &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_infoRequestResponse :
-      return ((const H225_InfoRequestResponse &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_nonStandardMessage :
-      return ((const H225_NonStandardMessage &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_unknownMessageResponse :
-      return ((const H225_UnknownMessageResponse &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_requestInProgress :
-      return ((const H225_RequestInProgress &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_resourcesAvailableIndicate :
-      return ((const H225_ResourcesAvailableIndicate &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_resourcesAvailableConfirm :
-      return ((const H225_ResourcesAvailableConfirm &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_infoRequestAck :
-      return ((const H225_InfoRequestAck &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_infoRequestNak :
-      return ((const H225_InfoRequestNak &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_serviceControlIndication :
-      return ((const H225_ServiceControlIndication &)*this).m_requestSeqNum;
-
-    case H225_RasMessage::e_serviceControlResponse :
-      return ((const H225_ServiceControlResponse &)*this).m_requestSeqNum;
-
-    default :
-      return 0;
-  }
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
