@@ -27,6 +27,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: ixjunix.cxx,v $
+ * Revision 1.136  2003/04/29 08:31:44  robertj
+ * Fixed return type of get wink function.
+ *
+ * Revision 1.135  2003/04/28 01:47:52  dereks
+ * Add ability to set/get wink duration for ixj device.
+ *
  * Revision 1.134  2002/11/05 04:44:20  robertj
  * Fixed typo
  *
@@ -1806,6 +1812,24 @@ BOOL OpalIxJDevice::SetAEC(unsigned line, AECLevels level)
 }
 
 
+unsigned OpalIxJDevice::GetWinkDuration(unsigned)
+{
+  if (!IsOpen())
+    return 0;
+
+  return IOCTL2(os_handle, IXJCTL_WINK_DURATION, 0);
+}
+
+
+BOOL OpalIxJDevice::SetWinkDuration(unsigned, unsigned winkDuration)
+{
+  if (!IsOpen())
+    return FALSE;  
+
+  return IOCTL2(os_handle, IXJCTL_WINK_DURATION, winkDuration);
+}
+
+
 BOOL OpalIxJDevice::GetVAD(unsigned)
 {
   return FALSE;
