@@ -24,6 +24,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: lid.cxx,v $
+ * Revision 1.96  2003/06/03 10:27:42  rjongbloed
+ * Added G.729 and G,729B detection from LID.
+ *
+ * Revision 1.95  2003/04/29 08:30:29  robertj
+ * Fixed return type of get wink function.
+ *
+ * Revision 1.94  2003/04/28 01:47:52  dereks
+ * Add ability to set/get wink duration for ixj device.
+ *
  * Revision 1.93  2003/03/05 06:26:44  robertj
  * Added function to play a WAV file to LID, thanks Pietro Ravasio
  *
@@ -686,6 +695,17 @@ BOOL OpalLineInterfaceDevice::SetAEC(unsigned, AECLevels)
   return FALSE;
 }
 
+unsigned OpalLineInterfaceDevice::GetWinkDuration(unsigned)
+{
+  return 0;
+}
+
+
+BOOL OpalLineInterfaceDevice::SetWinkDuration(unsigned, unsigned)
+{
+  return FALSE;
+}
+
 
 BOOL OpalLineInterfaceDevice::GetVAD(unsigned)
 {
@@ -1290,7 +1310,9 @@ static struct {
   { OPAL_G7231_5k3,    24,   8,  3, FALSE, H245_AudioCapability::e_g7231 },
   { OPAL_G7231_6k3,    24,   8,  3, FALSE, H245_AudioCapability::e_g7231 },
 
+  { OPAL_G729,         10,  24,  6, FALSE, H245_AudioCapability::e_g729 },
   { OPAL_G729A,        10,  24,  6, FALSE, H245_AudioCapability::e_g729AnnexA },
+  { OPAL_G729B,        10,  24,  6, FALSE, H245_AudioCapability::e_g729wAnnexB },
   { OPAL_G729AB,       10,  24,  6, FALSE, H245_AudioCapability::e_g729AnnexAwAnnexB },
   { OPAL_GSM0610,      33,   7,  4, FALSE, H245_AudioCapability::e_gsmFullRate },
   { OPAL_G728,          5,  96, 20, FALSE, H245_AudioCapability::e_g728 },
@@ -1319,11 +1341,21 @@ H323_REGISTER_CAPABILITY_FUNCTION(H323_LID_GSM0610_Capability, OPAL_GSM0610"{hw}
   return new H323_LIDCapability(OpalGSM0610);
 }
 
+H323_REGISTER_CAPABILITY_FUNCTION(H323_LID_G729_Capability, OPAL_G729"{hw}", H323_NO_EP_VAR)
+{
+  return new H323_LIDCapability(OpalG729);
+}
+
 H323_REGISTER_CAPABILITY_FUNCTION(H323_LID_G729A_Capability, OPAL_G729A"{hw}", H323_NO_EP_VAR)
 {
   return new H323_LIDCapability(OpalG729A);
 }
 
+H323_REGISTER_CAPABILITY_FUNCTION(H323_LID_G729B_Capability, OPAL_G729B"{hw}", H323_NO_EP_VAR)
+{
+  return new H323_LIDCapability(OpalG729B);
+}
+
 H323_REGISTER_CAPABILITY_FUNCTION(H323_LID_G729AB_Capability, OPAL_G729AB"{hw}", H323_NO_EP_VAR)
 {
   return new H323_LIDCapability(OpalG729AB);
