@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: q931.cxx,v $
+ * Revision 1.58  2003/03/18 05:54:18  robertj
+ * Added ChannelIdentifier IE support, thanks Eize Slange
+ *
  * Revision 1.57  2003/02/12 02:23:01  robertj
  * Fixed printing of cause codes that are impossible as opposed to merely
  *   unlikely or unallocated.
@@ -228,19 +231,20 @@
 ostream & operator<<(ostream & strm, Q931::InformationElementCodes ie)
 {
   static POrdinalToString::Initialiser IENamesInit[] = {
-    { Q931::BearerCapabilityIE,   "Bearer-Capability"     },
-    { Q931::CauseIE,              "Cause"                 },
-    { Q931::FacilityIE,           "Facility"              },
-    { Q931::ProgressIndicatorIE,  "Progress-Indicator"    },
-    { Q931::CallStateIE,          "Call-State"            },
-    { Q931::DisplayIE,            "Display"               },
-    { Q931::SignalIE,             "Signal"                },
-    { Q931::KeypadIE,             "Keypad"                },
-    { Q931::ConnectedNumberIE,    "Connected-Number"      },
-    { Q931::CallingPartyNumberIE, "Calling-Party-Number"  },
-    { Q931::CalledPartyNumberIE,  "Called-Party-Number"   },
-    { Q931::RedirectingNumberIE,  "Redirecting-Number"    },
-    { Q931::UserUserIE,           "User-User"             }
+    { Q931::BearerCapabilityIE,     "Bearer-Capability"     },
+    { Q931::CauseIE,                "Cause"                 },
+    { Q931::FacilityIE,             "Facility"              },
+    { Q931::ProgressIndicatorIE,    "Progress-Indicator"    },
+    { Q931::CallStateIE,            "Call-State"            },
+    { Q931::DisplayIE,              "Display"               },
+    { Q931::SignalIE,               "Signal"                },
+    { Q931::KeypadIE,               "Keypad"                },
+    { Q931::ConnectedNumberIE,      "Connected-Number"      },
+    { Q931::CallingPartyNumberIE,   "Calling-Party-Number"  },
+    { Q931::CalledPartyNumberIE,    "Called-Party-Number"   },
+    { Q931::RedirectingNumberIE,    "Redirecting-Number"    },
+    { Q931::ChannelIdentificationIE,"Channel-Identification"},
+    { Q931::UserUserIE,             "User-User"             } 
   };
   static const POrdinalToString IENames(PARRAYSIZE(IENamesInit), IENamesInit);
 
@@ -1178,4 +1182,103 @@ void Q931::SetConnectedNumber(const PString & number,
 }
 
 
+void Q931::SetChannelIdentification(unsigned interfaceType,
+                                    unsigned preferredOrExclusive,
+                                    int      channelNumber)
+{
+  // Known limitations:
+  //  - the interface identifier cannot be specified
+  //  - channel in PRI can only be indicated by number and cannot be indicated by map
+  //  - one and only one channel can be indicated
+  //  - the coding standard is always ITU Q.931
+
+  PBYTEArray bytes;
+  bytes.SetSize(1);
+
+  PAssert(interfaceType < 2, PInvalidParameter);
+
+  if (interfaceType == 0) { // basic rate
+    if (channelNumber == -1) { // any channel
+      bytes[0] = 0x80 | 0x04 | 0x03;
+    }
+    if (channelNumber == 0) { // D channel
+      bytes[0] = 0x80;
+    }    
+    if (channelNumber > 0) { // B channel
+      bytes[0] = (BYTE)(0x80 | 0x04 | ((preferredOrExclusive & 0x01) << 3) | (channelNumber & 0x03));
+    }
+  }
+
+  if (interfaceType == 1) { // primary rate
+    if (channelNumber == -1) { // any channel
+      bytes[0] = 0x80 | 0x20 | 0x04 | 0x03;
+    }
+    if (channelNumber == 0) { // D channel
+      bytes[0] = 0x80 | 0x20;
+    }    
+    if (channelNumber > 0) { // B channel
+      bytes.SetSize(3);
+
+      bytes[0] = (BYTE)(0x80 | 0x20 | 0x04 | ((preferredOrExclusive & 0x01) << 3) | 0x01);
+      bytes[1] = 0x80 | 0x03;
+      bytes[2] = (BYTE)(0x80 | channelNumber);
+    }
+  }
+
+  SetIE(ChannelIdentificationIE, bytes);
+}
+
+
+BOOL Q931::GetChannelIdentification(unsigned * interfaceType,
+                                    unsigned * preferredOrExclusive,
+                                    int      * channelNumber) const
+{
+  if (!HasIE(ChannelIdentificationIE))
+    return FALSE;
+
+  PBYTEArray bytes = GetIE(ChannelIdentificationIE);
+  if (bytes.GetSize() < 1)
+    return FALSE;
+
+  *interfaceType        = (bytes[0]>>5) & 0x01;
+  *preferredOrExclusive = (bytes[0]>>3) & 0x01;
+
+  if (*interfaceType == 0) {  // basic rate
+    if ( (bytes[0] & 0x04) == 0 ) {  // D Channel
+      *channelNumber = 0;
+    }
+    else {
+      if ( (bytes[0] & 0x03) == 0x03 ) {  // any channel
+        *channelNumber = -1;
+      }
+      else { // B Channel
+        *channelNumber = (bytes[0] & 0x03);
+      }
+    }
+  }
+
+  if (*interfaceType == 1) {  // primary rate
+    if ( (bytes[0] & 0x04) == 0 ) {  // D Channel
+      *channelNumber = 0;
+    }
+    else {
+      if ( (bytes[0] & 0x03) == 0x03 ) {  // any channel
+        *channelNumber = -1;
+      }
+      else { // B Channel
+        if (bytes.GetSize() < 3)
+          return FALSE;
+
+        if (bytes[1] != 0x83)
+          return FALSE;
+
+        *channelNumber = bytes[2] & 0x7f;
+      }
+    }
+  }
+
+  return TRUE;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
