@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: rtp.cxx,v $
+ * Revision 1.88  2003/05/02 04:57:47  robertj
+ * Added header extension support to RTP data frame class.
+ *
  * Revision 1.87  2003/02/07 00:30:21  robertj
  * Changes for bizarre usage of RTP code outside of scope of H.323 specs.
  *
@@ -346,11 +349,13 @@ const unsigned SecondsFrom1900to1970 = (70*365+17)*24*60*60U;
 
 #define UDP_BUFFER_SIZE 32768
 
+#define MIN_HEADER_SIZE 12
+
 
 /////////////////////////////////////////////////////////////////////////////
 
 RTP_DataFrame::RTP_DataFrame(PINDEX sz)
-  : PBYTEArray(12+sz)
+  : PBYTEArray(MIN_HEADER_SIZE+sz)
 {
   payloadSize = sz;
   theArray[0] = '\x80';
@@ -387,7 +392,7 @@ void RTP_DataFrame::SetPayloadType(PayloadTypes t)
 DWORD RTP_DataFrame::GetContribSource(PINDEX idx) const
 {
   PAssert(idx < GetContribSrcCount(), PInvalidParameter);
-  return ((PUInt32b *)&theArray[12])[idx];
+  return ((PUInt32b *)&theArray[MIN_HEADER_SIZE])[idx];
 }
 
 
@@ -403,7 +408,68 @@ void RTP_DataFrame::SetContribSource(PINDEX idx, DWORD src)
     memmove(GetPayloadPtr(), oldPayload, payloadSize);
   }
 
-  ((PUInt32b *)&theArray[12])[idx] = src;
+  ((PUInt32b *)&theArray[MIN_HEADER_SIZE])[idx] = src;
+}
+
+
+PINDEX RTP_DataFrame::GetHeaderSize() const
+{
+  PINDEX sz = MIN_HEADER_SIZE + 4*GetContribSrcCount();
+
+  if (GetExtension())
+    sz += 4 + GetExtensionSize();
+
+  return sz;
+}
+
+
+int RTP_DataFrame::GetExtensionType() const
+{
+  if (GetExtension())
+    return *(PUInt16b *)&theArray[MIN_HEADER_SIZE + 4*GetContribSrcCount()];
+
+  return -1;
+}
+
+
+void RTP_DataFrame::SetExtensionType(int type)
+{
+  if (type < 0)
+    SetExtension(FALSE);
+  else {
+    if (!GetExtension())
+      SetExtensionSize(0);
+    *(PUInt16b *)&theArray[MIN_HEADER_SIZE + 4*GetContribSrcCount()] = (WORD)type;
+  }
+}
+
+
+PINDEX RTP_DataFrame::GetExtensionSize() const
+{
+  if (GetExtension())
+    return *(PUInt16b *)&theArray[MIN_HEADER_SIZE + 4*GetContribSrcCount() + 2];
+
+  return 0;
+}
+
+
+BOOL RTP_DataFrame::SetExtensionSize(PINDEX sz)
+{
+  if (!SetMinSize(MIN_HEADER_SIZE + 4*GetContribSrcCount() + 4+4*sz + payloadSize))
+    return FALSE;
+
+  SetExtension(TRUE);
+  *(PUInt16b *)&theArray[MIN_HEADER_SIZE + 4*GetContribSrcCount() + 2] = (WORD)sz;
+  return TRUE;
+}
+
+
+BYTE * RTP_DataFrame::GetExtensionPtr() const
+{
+  if (GetExtension())
+    return (BYTE *)&theArray[MIN_HEADER_SIZE + 4*GetContribSrcCount() + 4];
+
+  return NULL;
 }
 
 
