@@ -27,6 +27,28 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: transports.cxx,v $
+ * Revision 1.134  2003/04/10 09:45:34  robertj
+ * Added associated transport to new GetInterfaceAddresses() function so
+ *   interfaces can be ordered according to active transport links. Improves
+ *   interoperability.
+ * Replaced old listener GetTransportPDU() with GetInterfaceAddresses()
+ *   and H323SetTransportAddresses() functions.
+ *
+ * Revision 1.133  2003/04/10 00:58:54  craigs
+ * Added functions to access to lists of interfaces
+ *
+ * Revision 1.132  2003/03/26 06:14:31  robertj
+ * More IPv6 support (INADDR_ANY handling), thanks Sbastien Josset
+ *
+ * Revision 1.131  2003/03/21 05:24:54  robertj
+ * Added setting of remote port in UDP transport constructor.
+ *
+ * Revision 1.130  2003/03/20 01:51:12  robertj
+ * More abstraction of H.225 RAS and H.501 protocols transaction handling.
+ *
+ * Revision 1.129  2003/03/11 23:15:23  robertj
+ * Fixed possible double delete of socket (crash) on garbage input.
+ *
  * Revision 1.128  2003/02/06 04:31:02  robertj
  * Added more support for adding things to H323TransportAddressArrays
  *
@@ -739,7 +761,7 @@ BOOL H323TransportAddress::IsEquivalent(const H323TransportAddress & address)
   WORD port1 = 65535, port2 = 65535;
   return GetIpAndPort(ip1, port1) &&
          address.GetIpAndPort(ip2, port2) &&
-         (ip1 == INADDR_ANY || ip2 == INADDR_ANY || ip1 == ip2) &&
+         (ip1.IsAny() || ip2.IsAny() || ip1 == ip2) &&
          (port1 == 65535 || port2 == 65535 || port1 == port2);
 }
 
@@ -803,7 +825,7 @@ BOOL H323TransportAddress::GetIpAndPort(PIPSocket::Address & ip,
   }
 
   if (host == "*") {
-    ip = INADDR_ANY;
+    ip = PIPSocket::GetDefaultIpAny();
     return TRUE;
   }
 
@@ -880,6 +902,108 @@ H323Transport * H323TransportAddress::CreateTransport(H323EndPoint & endpoint) c
 }
 
 
+H323TransportAddressArray H323GetInterfaceAddresses(const H323ListenerList & listeners,
+                                                    BOOL excludeLocalHost,
+                                                    H323Transport * associatedTransport)
+{
+  H323TransportAddressArray interfaceAddresses;
+
+  PINDEX i;
+  for (i = 0; i < listeners.GetSize(); i++) {
+    H323TransportAddressArray newAddrs = H323GetInterfaceAddresses(listeners[i].GetTransportAddress(), excludeLocalHost, associatedTransport);
+    PINDEX size  = interfaceAddresses.GetSize();
+    PINDEX nsize = newAddrs.GetSize();
+    interfaceAddresses.SetSize(size + nsize);
+    PINDEX j;
+    for (j = 0; j < nsize; j++)
+      interfaceAddresses.SetAt(size + j, new H323TransportAddress(newAddrs[j]));
+  }
+
+  return interfaceAddresses;
+}
+
+
+H323TransportAddressArray H323GetInterfaceAddresses(const H323TransportAddress & addr,
+                                                    BOOL excludeLocalHost,
+                                                    H323Transport * associatedTransport)
+{
+  PIPSocket::Address ip;
+  WORD port;
+  if (!addr.GetIpAndPort(ip, port) || !ip.IsAny())
+    return addr;
+
+  PIPSocket::InterfaceTable interfaces;
+  if (!PIPSocket::GetInterfaceTable(interfaces))
+    return addr;
+
+  if (interfaces.GetSize() == 1)
+    return H323TransportAddress(interfaces[0].GetAddress(), port);
+
+  PINDEX i;
+  H323TransportAddressArray interfaceAddresses;
+  PIPSocket::Address firstAddress(0);
+
+  if (associatedTransport != NULL) {
+    if (associatedTransport->GetLocalAddress().GetIpAddress(firstAddress)) {
+      for (i = 0; i < interfaces.GetSize(); i++) {
+        PIPSocket::Address ip = interfaces[i].GetAddress();
+        if (ip == firstAddress)
+          interfaceAddresses.Append(new H323TransportAddress(ip, port));
+      }
+    }
+  }
+
+  for (i = 0; i < interfaces.GetSize(); i++) {
+    PIPSocket::Address ip = interfaces[i].GetAddress();
+    if (ip != firstAddress && !(excludeLocalHost && ip.IsLoopback()))
+      interfaceAddresses.Append(new H323TransportAddress(ip, port));
+  }
+
+  return interfaceAddresses;
+}
+
+
+void H323SetTransportAddresses(const H323Transport & associatedTransport,
+                               const H323TransportAddressArray & addresses,
+                               H225_ArrayOf_TransportAddress & pdu)
+{
+  for (PINDEX i = 0; i < addresses.GetSize(); i++) {
+    H323TransportAddress addr = addresses[i];
+
+    PTRACE(4, "TCP\tAppending H.225 transport " << addr
+           << " using associated transport " << associatedTransport);
+
+    PIPSocket::Address ip;
+    WORD port;
+    if (addr.GetIpAndPort(ip, port)) {
+      PIPSocket::Address remoteIP;
+      if (associatedTransport.GetRemoteAddress().GetIpAddress(remoteIP)) {
+        associatedTransport.GetEndPoint().TranslateTCPAddress(ip, remoteIP);
+        addr = H323TransportAddress(ip, port);
+      }
+    }
+
+    H225_TransportAddress pduAddr;
+    addr.SetPDU(pduAddr);
+
+    PINDEX lastPos = pdu.GetSize();
+
+    // Check for already have had that address.
+    PINDEX j;
+    for (j = 0; j < lastPos; j++) {
+      if (pdu[j] == pduAddr)
+        break;
+    }
+
+    if (j >= lastPos) {
+      // Put new listener into array
+      pdu.SetSize(lastPos+1);
+      pdu[lastPos] = pduAddr;
+    }
+  }
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 H323TransportAddressArray::H323TransportAddressArray(const H225_ArrayOf_TransportAddress & addresses)
@@ -1170,8 +1294,10 @@ H323Transport * H323ListenerTCP::Accept(const PTimeInterval & timeout)
 
     PTRACE(1, "TCP\tFailed to open transport, connection not started.");
     delete transport;
+    return NULL;
   }
-  else if (socket->GetErrorCode() != PChannel::Interrupted) {
+
+  if (socket->GetErrorCode() != PChannel::Interrupted) {
     PTRACE(1, "TCP\tAccept error:" << socket->GetErrorText());
     listener.Close();
   }
@@ -1187,77 +1313,10 @@ H323TransportAddress H323ListenerTCP::GetTransportAddress() const
 }
 
 
-static void AppendTransportAddress(H323EndPoint & endpoint,
-                                   const H323Transport & associatedTransport,
-                                   PIPSocket::Address addr, WORD port,
-                                   H225_ArrayOf_TransportAddress & pdu)
-{
-  PTRACE(4, "TCP\tAppending H.225 transport " << addr << ':' << port
-         << " using associated transport " << associatedTransport);
-
-  PIPSocket::Address remoteIP;
-  if (associatedTransport.GetRemoteAddress().GetIpAddress(remoteIP))
-    endpoint.TranslateTCPAddress(addr, remoteIP);
-
-  H323TransportAddress transAddr(addr, port);
-  H225_TransportAddress pduAddr;
-  transAddr.SetPDU(pduAddr);
-
-  PINDEX lastPos = pdu.GetSize();
-
-  // Check for already have had that address.
-  for (PINDEX i = 0; i < lastPos; i++) {
-    if (pdu[i] == pduAddr)
-      return;
-  }
-
-  // Put new listener into array
-  pdu.SetSize(lastPos+1);
-  pdu[lastPos] = pduAddr;
-}
-
-
-void H323ListenerTCP::SetUpTransportPDU(H225_ArrayOf_TransportAddress & pdu,
-                                        const H323Transport & associatedTransport)
-{
-  WORD port = listener.GetPort();
-
-  if (localAddress != INADDR_ANY) {
-    AppendTransportAddress(endpoint, associatedTransport, localAddress, port, pdu);
-    return;
-  }
-
-  PIPSocket::InterfaceTable interfaces;
-  if (!PIPSocket::GetInterfaceTable(interfaces)) {
-    PIPSocket::Address ipAddr;
-    PIPSocket::GetHostAddress(ipAddr);
-    AppendTransportAddress(endpoint, associatedTransport, ipAddr, port, pdu);
-    return;
-  }
-
-  PINDEX i;
-
-  PIPSocket::Address firstAddress;
-  if (associatedTransport.GetLocalAddress().GetIpAddress(firstAddress)) {
-    for (i = 0; i < interfaces.GetSize(); i++) {
-      PIPSocket::Address ipAddr = interfaces[i].GetAddress();
-      if (ipAddr == firstAddress)
-        AppendTransportAddress(endpoint, associatedTransport, ipAddr, port, pdu);
-    }
-  }
-
-  for (i = 0; i < interfaces.GetSize(); i++) {
-    PIPSocket::Address ipAddr = interfaces[i].GetAddress();
-    if (ipAddr != 0 && ipAddr != firstAddress && !ipAddr.IsLoopback()) // Ignore 127.0.0.1
-      AppendTransportAddress(endpoint, associatedTransport, ipAddr, port, pdu);
-  }
-}
-
-
 BOOL H323ListenerTCP::SetUpTransportPDU(H245_TransportAddress & pdu,
                                         const H323Transport & associatedTransport)
 {
-  if (localAddress != INADDR_ANY)
+  if (!localAddress.IsAny())
     return GetTransportAddress().SetPDU(pdu);
 
   PIPSocket::Address addressOfExistingInterface;
@@ -1642,15 +1701,19 @@ static BOOL ListenUDP(PUDPSocket & socket,
 }
 
 
-H323TransportUDP::H323TransportUDP(H323EndPoint & end,
+H323TransportUDP::H323TransportUDP(H323EndPoint & ep,
                                    PIPSocket::Address binding,
-                                   WORD port)
-  : H323TransportIP(end, binding, H225_RAS::DefaultRasUdpPort)
+                                   WORD local_port,
+                                   WORD remote_port)
+  : H323TransportIP(ep, binding, remote_port)
 {
+  if (remotePort == 0)
+    remotePort = H225_RAS::DefaultRasUdpPort; // For backward compatibility
+
   promiscuousReads = AcceptFromRemoteOnly;
 
   PUDPSocket * udp = new PUDPSocket;
-  ListenUDP(*udp, end, binding, port);
+  ListenUDP(*udp, ep, binding, local_port);
 
   localPort = udp->GetPort();
 
@@ -1948,7 +2011,7 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
     }
 
     do {
-      if (gk.HandleRasPDU(response)) {
+      if (gk.HandleTransaction(response)) {
         if (!gk.IsDiscoveryComplete()) {
           localAddress = originalLocalAddress;
           localPort = originalLocalPort;
