@@ -26,6 +26,9 @@
  * Changed so only BUSY tone reports line hangup/busy.
  *
  * $Log: vpblid.cxx,v $
+ * Revision 1.20  2003/03/12 00:15:40  dereks
+ * Fix compile error on RH8.0
+ *
  * Revision 1.19  2003/03/05 06:26:44  robertj
  * Added function to play a WAV file to LID, thanks Pietro Ravasio
  *
@@ -625,7 +628,7 @@ BOOL OpalVpbDevice::StopTone(unsigned line)
 BOOL OpalVpbDevice::PlayAudio(unsigned line, const PString & fn)
 {
   PTRACE(3, "VPB\tPlayAudio starting a new Audio Thread on line " << line << " with file " << fn);
-  vpb_play_file_async(lineState[line].handle, fn.GetPointer(), VPB_PLAYEND);
+  vpb_play_file_async(lineState[line].handle, (char *)&fn, VPB_PLAYEND);
   /*
   lineState[line].myAudioThread = new AudioThread(
   */
